@@ -9,58 +9,79 @@
 #'
 #' @examples
 #' # U Island example region
-#' coordinates <- data.frame(x = rep(seq(177.01, 177.05, 0.01), 5),
-#'                           y = rep(seq(-18.01, -18.05, -0.01), each = 5))
-#' coordinates <- coordinates[c(7, 9, 12, 14, 17:19),]
+#' coordinates <- data.frame(
+#'   x = rep(seq(177.01, 177.05, 0.01), 5),
+#'   y = rep(seq(-18.01, -18.05, -0.01), each = 5)
+#' )
+#' coordinates <- coordinates[c(7, 9, 12, 14, 17:19), ]
 #' region <- Region$new(coordinates = coordinates, use_raster = FALSE)
 #' # Spatial correlation
-#' spatial_correlation <- SpatialCorrelation$new(region = region, correlation_amplitude = 0.6,
-#'                                               correlation_breadth = 300)
+#' spatial_correlation <- SpatialCorrelation$new(
+#'   region = region, correlation_amplitude = 0.6,
+#'   correlation_breadth = 300
+#' )
 #' spatial_correlation$calculate_compact_decomposition(decimals = 4)
 #' # Example habitat suitability in file
-#' saveRDS(array(c(0.5, 0.3, 0.7, 0.9, 0.6, 0.7, 0.8), c(7, 5)),
-#'         file.path(tempdir(), "hs_mean_1.RData"))
+#' saveRDS(
+#'   array(c(0.5, 0.3, 0.7, 0.9, 0.6, 0.7, 0.8), c(7, 5)),
+#'   file.path(tempdir(), "hs_mean_1.RData")
+#' )
 #' # Generator
-#' capacity_gen <- Generator$new(description = "capacity",
-#'                               region = region,
-#'                               time_steps = 5,
-#'                               spatial_correlation = spatial_correlation,
-#'                               temporal_correlation = 0.9,
-#'                               hs_sd = 0.1, # template attached
-#'                               inputs = c("hs_file", "density_max", "initial_n"),
-#'                               outputs = c("initial_abundance", "carrying_capacity"))
-#' capacity_gen$add_generative_requirements(list(hs_mean = "file",
-#'                                               hs_sample = "distribution",
-#'                                               carrying_capacity = "function",
-#'                                               initial_abundance = "function"))
+#' capacity_gen <- Generator$new(
+#'   description = "capacity",
+#'   region = region,
+#'   time_steps = 5,
+#'   spatial_correlation = spatial_correlation,
+#'   temporal_correlation = 0.9,
+#'   hs_sd = 0.1, # template attached
+#'   inputs = c("hs_file", "density_max", "initial_n"),
+#'   outputs = c("initial_abundance", "carrying_capacity")
+#' )
+#' capacity_gen$add_generative_requirements(list(
+#'   hs_mean = "file",
+#'   hs_sample = "distribution",
+#'   carrying_capacity = "function",
+#'   initial_abundance = "function"
+#' ))
 #' # File template for mean habitat suitability
 #' capacity_gen$add_file_template("hs_mean",
-#'                                path_template = file.path(tempdir(), "hs_mean_%s.RData"),
-#'                                path_params = c("hs_file"), file_type = "RDS")
+#'   path_template = file.path(tempdir(), "hs_mean_%s.RData"),
+#'   path_params = c("hs_file"), file_type = "RDS"
+#' )
 #' # Distribution template for sampling habitat suitability
 #' capacity_gen$add_distribution_template("hs_sample",
-#'                                        distr_type = "beta",
-#'                                        distr_params = list(mean = "hs_mean",
-#'                                                            sd = "hs_sd"))
+#'   distr_type = "beta",
+#'   distr_params = list(
+#'     mean = "hs_mean",
+#'     sd = "hs_sd"
+#'   )
+#' )
 #' # Function templates for initial abundance and carrying capacity
 #' capacity_gen$add_function_template("initial_abundance",
-#'                                    function_def = function(params) {
-#'                                      stats::rmultinom(1, size = params$initial_n,
-#'                                                       prob = params$hs_sample[,1])
-#'                                    },
-#'                                    call_params = c("initial_n", "hs_sample"))
+#'   function_def = function(params) {
+#'     stats::rmultinom(1,
+#'       size = params$initial_n,
+#'       prob = params$hs_sample[, 1]
+#'     )
+#'   },
+#'   call_params = c("initial_n", "hs_sample")
+#' )
 #' capacity_gen$add_function_template("carrying_capacity",
-#'                                    function_def = function(params) {
-#'                                      round(params$density_max*params$hs_sample)
-#'                                    },
-#'                                    call_params = c("density_max", "hs_sample"))
+#'   function_def = function(params) {
+#'     round(params$density_max * params$hs_sample)
+#'   },
+#'   call_params = c("density_max", "hs_sample")
+#' )
 #' # Generation
-#' capacity_gen$generate(input_values = list(hs_file = 1,
-#'                                           initial_n = 400,
-#'                                           density_max = 100))
+#' capacity_gen$generate(input_values = list(
+#'   hs_file = 1,
+#'   initial_n = 400,
+#'   density_max = 100
+#' ))
 #'
 #' @importFrom R6 R6Class
 #' @importFrom metRology qtri
+#' @importFrom qs qread
 #' @include SpatialModel.R
 #' @include GenerativeTemplate.R
 #' @export Generator
@@ -120,8 +141,10 @@ Generator <- R6Class("Generator",
     #' @param ... Parameters passed via the inherited class constructor (defined in initialize and run via new).
     #' @return New object of the current (inherited) class.
     new_clone = function(...) {
-      return(super$new_clone(generative_template = self$generative_template,
-                             generative_requirements = self$generative_requirements, ...))
+      return(super$new_clone(
+        generative_template = self$generative_template,
+        generative_requirements = self$generative_requirements, ...
+      ))
     },
 
     #' @description
@@ -138,7 +161,7 @@ Generator <- R6Class("Generator",
         for (param in params[!params %in% names(attribute_list)]) {
           required_type <- self$generative_requirements[[param]]
           if (param %in% names(self$attribute_aliases) && is.null(required_type) &&
-              !(param %in% c(names(self$file_templates), names(self$function_templates), names(self$distribution_templates)))) {
+            !(param %in% c(names(self$file_templates), names(self$function_templates), names(self$distribution_templates)))) {
             attribute <- self$attribute_aliases[[param]]
             attribute_root <- unlist(strsplit(attribute, "$", fixed = TRUE))[1]
             attribute_root <- unlist(strsplit(attribute_root, "[", fixed = TRUE))[1]
@@ -147,26 +170,26 @@ Generator <- R6Class("Generator",
             attribute_root <- attribute <- param
           }
           if ((is.null(required_type) || required_type == "file") && attribute_root %in% names(self$file_templates)) {
-            eval(parse(text=sprintf("%s <- self$read_file(attribute_root)", attribute_root)))
-            attribute_list[[param]] <- eval(parse(text=attribute))
+            eval(parse(text = sprintf("%s <- self$read_file(attribute_root)", attribute_root)))
+            attribute_list[[param]] <- eval(parse(text = attribute))
           } else if ((is.null(required_type) || required_type == "function") && attribute_root %in% names(self$function_templates)) {
-            eval(parse(text=sprintf("%s <- self$run_function(attribute_root)", attribute_root)))
-            attribute_list[[param]] <- eval(parse(text=attribute))
+            eval(parse(text = sprintf("%s <- self$run_function(attribute_root)", attribute_root)))
+            attribute_list[[param]] <- eval(parse(text = attribute))
           } else if ((is.null(required_type) || required_type == "distribution") && attribute_root %in% names(self$distribution_templates)) {
-            eval(parse(text=sprintf("%s <- self$sample_distribution(attribute_root)", attribute_root)))
-            attribute_list[[param]] <- eval(parse(text=attribute))
+            eval(parse(text = sprintf("%s <- self$sample_distribution(attribute_root)", attribute_root)))
+            attribute_list[[param]] <- eval(parse(text = attribute))
           }
         }
       }
       for (param in names(attribute_list)) {
         # Check consistency of any rasters
         if (!is.null(self$region) && any(class(attribute_list[[param]])[1] %in% c("RasterLayer", "RasterStack", "RasterBrick")) &&
-            !self$region$raster_is_consistent(attribute_list[[param]])) {
+          !self$region$raster_is_consistent(attribute_list[[param]])) {
           self$error_messages <- paste("Region is inconsistent with template generated raster for attribute: ", param)
         }
         if (!is.null(attribute_list[[param]]) && any(c(param, self$attribute_aliases[[param]]) %in% self$outputs) &&
-            (is.numeric(attribute_list[[param]]) ||
-             class(attribute_list[[param]])[1] %in% c("data.frame", "RasterLayer", "RasterStack", "RasterBrick"))) {
+          (is.numeric(attribute_list[[param]]) ||
+            class(attribute_list[[param]])[1] %in% c("data.frame", "RasterLayer", "RasterStack", "RasterBrick"))) {
           # Apply decimal rounding
           if (!is.null(self$decimals)) {
             attribute_list[[param]][] <- round(attribute_list[[param]][], self$decimals)
@@ -187,11 +210,11 @@ Generator <- R6Class("Generator",
           if (!is.null(self$occupancy_mask)) {
             if (length(as.matrix(attribute_list[[param]][])) < length(as.matrix(self$occupancy_mask[]))) {
               if (nrow(as.matrix(attribute_list[[param]][])) == nrow(as.matrix(self$occupancy_mask[])) &&
-                  ncol(as.matrix(attribute_list[[param]][])) == 1) { # use first column of mask
-                attribute_list[[param]][] <- attribute_list[[param]][]*self$occupancy_mask[][, 1]
+                ncol(as.matrix(attribute_list[[param]][])) == 1) { # use first column of mask
+                attribute_list[[param]][] <- attribute_list[[param]][] * self$occupancy_mask[][, 1]
               } # else don't apply mask
             } else {
-              attribute_list[[param]][] <- as.matrix(attribute_list[[param]][]*self$occupancy_mask[])
+              attribute_list[[param]][] <- as.matrix(attribute_list[[param]][] * self$occupancy_mask[])
             }
           }
         }
@@ -232,21 +255,28 @@ Generator <- R6Class("Generator",
     },
 
     #' @description
-    #' Adds a file template for reading raster/RData(RDS)/CSV files for a given model attribute.
+    #' Adds a file template for reading raster/RData(RDS)/CSV files for a given
+    #'  model attribute.
     #' @param param Name of model attribute to be read from a file.
-    #' @param path_template Template string for the file path with \emph{\%s} placeholders (see \code{\link{sprintf}}) for simulation sample parameters.
-    #' @param path_params Array of the names of the simulation sample parameters to be substituted (in order) into the path template.
-    #' @param file_type File type raster \emph{"GRD"} (default), \emph{"RData/RDS"} or \emph{"CSV"} to be read.
+    #' @param path_template Template string for the file path with placeholders
+    #'  (see \code{\link{sprintf}}) for simulation sample parameters.
+    #' @param path_params Array of the names of the simulation sample 
+    #' parameters to be substituted (in order) into the path template.
+    #' @param file_type File type raster \emph{"GRD"} (default), \emph{"TIF"},
+    #'  \emph{"RData/RDS"}, \emph{"QS"}, or \emph{"CSV"} to be read.
     add_file_template = function(param, path_template, path_params = c(), file_type = "GRD") {
       if (is.character(param) && is.character(path_template)) {
-        if (length(path_params) == (length(strsplit(paste0("#", path_template,"#"), "%s")[[1]]) - 1)) {
+        if (length(path_params) == (length(strsplit(paste0("#", path_template, "#"), "%s")[[1]]) - 1)) {
           self$file_templates[[param]] <- list()
           self$file_templates[[param]]$path_template <- path_template
           self$file_templates[[param]]$path_params <- c(path_params)
-          if (toupper(file_type) == "GRD" || toupper(file_type) == "RDS" || toupper(file_type) == "CSV") {
+          if (toupper(file_type) == "GRD" || toupper(file_type) == "RDS" || 
+          toupper(file_type) == "CSV" || toupper(file_type) == "TIF" || 
+          toupper(file_type) == "QS"
+          ) {
             self$file_templates[[param]]$file_type <- toupper(file_type)
           } else {
-            stop("The file type should be GRD (raster), RDS or CSV", call. = FALSE)
+            stop("The file type should be GRD (raster), TIF (raster), RDS, QS, or CSV", call. = FALSE)
           }
         } else {
           stop("Ensure the path template contains a corresponding %s for each path parameter", call. = FALSE)
@@ -268,19 +298,23 @@ Generator <- R6Class("Generator",
       # Assign function definition from a file or directly
       if (is.character(function_def)) {
         if (file.exists(function_def) && length(grep(".R", toupper(function_def), fixed = TRUE))) { # from file
-          tryCatch({
-            function_def <- source(function_def)$value # direct assignment from a file
-          },
-          error = function(e){
-            stop(paste("Error loading function from file", function_def, ":", as.character(e)), call. = FALSE)
-          })
+          tryCatch(
+            {
+              function_def <- source(function_def)$value # direct assignment from a file
+            },
+            error = function(e) {
+              stop(paste("Error loading function from file", function_def, ":", as.character(e)), call. = FALSE)
+            }
+          )
         } else { # named function
-          tryCatch({
-            function_def <- eval(parse(text = function_def))
-          },
-          error = function(e){
-            stop(paste("Could not assign function", function_def, ":", as.character(e)), call. = FALSE)
-          })
+          tryCatch(
+            {
+              function_def <- eval(parse(text = function_def))
+            },
+            error = function(e) {
+              stop(paste("Could not assign function", function_def, ":", as.character(e)), call. = FALSE)
+            }
+          )
         }
       }
       if (!is.function(function_def)) {
@@ -308,16 +342,19 @@ Generator <- R6Class("Generator",
       }
       distr_type <- match.arg(distr_type)
       expected_distr_params <- switch(distr_type,
-                                      uniform = c("lower", "upper"),
-                                      normal = c("mean", "sd"),
-                                      lognormal = list(c("meanlog", "sdlog"), c("mean", "sd")),
-                                      beta = list(c("alpha", "beta"), c("mean", "sd")),
-                                      triangular = c("lower", "mode", "upper"))
+        uniform = c("lower", "upper"),
+        normal = c("mean", "sd"),
+        lognormal = list(c("meanlog", "sdlog"), c("mean", "sd")),
+        beta = list(c("alpha", "beta"), c("mean", "sd")),
+        triangular = c("lower", "mode", "upper")
+      )
       if (is.list(expected_distr_params)) {
         if (!is.list(distr_params) || !(all(expected_distr_params[[1]] %in% names(distr_params)) ||
-                                        all(expected_distr_params[[2]] %in% names(distr_params)))) {
-          stop(sprintf("The distribution parameters should be a list containing %s distribution parameters: %s (or %s)", distr_type,
-                       paste(expected_distr_params[[1]], collapse = ", "), paste(expected_distr_params[[2]], collapse = ", ")), call. = FALSE)
+          all(expected_distr_params[[2]] %in% names(distr_params)))) {
+          stop(sprintf(
+            "The distribution parameters should be a list containing %s distribution parameters: %s (or %s)", distr_type,
+            paste(expected_distr_params[[1]], collapse = ", "), paste(expected_distr_params[[2]], collapse = ", ")
+          ), call. = FALSE)
         }
       } else if (!is.list(distr_params) || !all(expected_distr_params %in% names(distr_params))) {
         stop(sprintf("The distribution parameters should be a list containing %s distribution parameters: %s", distr_type, paste(expected_distr_params, collapse = ", ")), call. = FALSE)
@@ -328,11 +365,13 @@ Generator <- R6Class("Generator",
       if (!is.null(random_seed) && !is.numeric(random_seed)) {
         stop("The random seed should be numeric", call. = FALSE)
       }
-      self$distribution_templates[[param]] <- list(distr_type = distr_type,
-                                                   distr_params = distr_params,
-                                                   sample = sample,
-                                                   random_seed = random_seed,
-                                                   normalize_threshold = normalize_threshold)
+      self$distribution_templates[[param]] <- list(
+        distr_type = distr_type,
+        distr_params = distr_params,
+        sample = sample,
+        random_seed = random_seed,
+        normalize_threshold = normalize_threshold
+      )
     },
 
     #' @description
@@ -352,6 +391,8 @@ Generator <- R6Class("Generator",
               value_list[[param]] <- utils::read.csv(file = file_path)
             } else if (self$file_templates[[param]]$file_type == "RDS") { # RDS (RData)
               value_list[[param]] <- readRDS(file = file_path)
+            } else if (self$file_templates[[param]]$file_type == "QS") {
+              value_list[[param]] <- qread(file = file_path)
             } else { # raster
               value_list[[param]] <- raster::brick(file_path)
             }
@@ -400,7 +441,6 @@ Generator <- R6Class("Generator",
     #' @return Model attribute value calculated via distribution sampling.
     sample_distribution = function(param) {
       if (param %in% names(self$distribution_templates)) {
-
         # Unpack template parameters
         distr_type <- self$distribution_templates[[param]][["distr_type"]]
         distr_params <- self$distribution_templates[[param]][["distr_params"]]
@@ -410,10 +450,12 @@ Generator <- R6Class("Generator",
           if (is.character(sample$mid)) sample$mid <- self$get_attribute(sample$mid)
           if (is.character(sample$window)) sample$window <- self$get_attribute(sample$window)
           if (is.numeric(sample$mid) && is.numeric(sample$window)) {
-            sample <- c(max(sample$mid - sample$window/2, 0), min(sample$mid + sample$window/2, 1))
+            sample <- c(max(sample$mid - sample$window / 2, 0), min(sample$mid + sample$window / 2, 1))
           } else {
-            self$error_messages <- sprintf("The distribution sample for %s utilizes missing parameter(s): %s", param,
-                                           paste(unlist(self$distribution_templates[[param]][["sample"]][which(!sample_names %in% names(sample))]), collapse = ", "))
+            self$error_messages <- sprintf(
+              "The distribution sample for %s utilizes missing parameter(s): %s", param,
+              paste(unlist(self$distribution_templates[[param]][["sample"]][which(!sample_names %in% names(sample))]), collapse = ", ")
+            )
             return(NULL)
           }
         } else if (!is.null(sample)) { # single or c(lower, upper) bounds
@@ -429,8 +471,10 @@ Generator <- R6Class("Generator",
           }
           sample <- suppressWarnings(as.numeric(sample))
           if (any(is.na(sample))) {
-            self$error_messages <- sprintf("The distribution sample for %s utilizes missing parameter(s): %s", param,
-                                           paste(self$distribution_templates[[param]][["sample"]][which(is.na(sample))]))
+            self$error_messages <- sprintf(
+              "The distribution sample for %s utilizes missing parameter(s): %s", param,
+              paste(self$distribution_templates[[param]][["sample"]][which(is.na(sample))])
+            )
             return(NULL)
           }
         }
@@ -456,9 +500,11 @@ Generator <- R6Class("Generator",
         if (self$uses_correlations) {
           # Use a spatial correlation object to generate correlated normal deviates using the random seed when present
           if (self$temporal_correlation < 1) {
-            correlated_deviates <- self$spatial_correlation$generate_correlated_normal_deviates(random_seed = random_seed,
-                                                                                              temporal_correlation = self$temporal_correlation,
-                                                                                              time_steps = self$time_steps)
+            correlated_deviates <- self$spatial_correlation$generate_correlated_normal_deviates(
+              random_seed = random_seed,
+              temporal_correlation = self$temporal_correlation,
+              time_steps = self$time_steps
+            )
           } else {
             correlated_deviates <- self$spatial_correlation$generate_correlated_normal_deviates(random_seed = random_seed)
           }
@@ -468,7 +514,7 @@ Generator <- R6Class("Generator",
           }
           # Convert normal correlated deviates to correlated uniform samples between lower and upper values if present
           if (length(sample) > 1) {
-            generated_samples <- sample[1] + pnorm(correlated_deviates)*(sample[2] - sample[1])
+            generated_samples <- sample[1] + pnorm(correlated_deviates) * (sample[2] - sample[1])
           } else { # use 0-1
             generated_samples <- pnorm(correlated_deviates)
           }
@@ -477,7 +523,7 @@ Generator <- R6Class("Generator",
             generated_samples <- sample
           } else {
             if (is.null(sample)) {
-              sample = c(0, 1)
+              sample <- c(0, 1)
             }
             if (!is.null(random_seed)) {
               set.seed(random_seed)
@@ -500,7 +546,7 @@ Generator <- R6Class("Generator",
           }
           if (!is.null(distr_param_values[[distr_param]])) {
             if (!is.null(self$region) && self$region$use_raster &&
-                any(class(distr_param_values[[distr_param]]) %in% c("RasterLayer", "RasterStack", "RasterBrick"))) {
+              any(class(distr_param_values[[distr_param]]) %in% c("RasterLayer", "RasterStack", "RasterBrick"))) {
               distr_param_values[[distr_param]] <- distr_param_values[[distr_param]][self$region$region_indices]
             }
             if (distr_type %in% c("uniform", "triangular")) {
@@ -514,92 +560,99 @@ Generator <- R6Class("Generator",
         # Ensure all distribution parameters have values
         if (!all(names(distr_params) %in% names(distr_param_values))) {
           missing_values <- names(distr_params)[which(!names(distr_params) %in% names(distr_param_values))]
-          self$error_messages <- sprintf("Values for %s distribution not obtained for parameters: %s (via %s)",
-                                         distr_type, paste(missing_values, collapse = ", "),
-                                         paste(unlist(distr_params[missing_values]), collapse = ", "))
+          self$error_messages <- sprintf(
+            "Values for %s distribution not obtained for parameters: %s (via %s)",
+            distr_type, paste(missing_values, collapse = ", "),
+            paste(unlist(distr_params[missing_values]), collapse = ", ")
+          )
           return(NULL)
         }
 
         # Generate attribute sampled from distribution
         switch(distr_type,
-               uniform = generated_values <- stats::qunif(generated_samples,
-                                                          min = distr_param_values$lower[],
-                                                          max = distr_param_values$upper[]),
-               normal = generated_values <- stats::qnorm(generated_samples,
-                                                         mean = distr_param_values$mean[],
-                                                         sd = distr_param_values$sd[]),
-               lognormal =
-                 {
-                   if (all(c("mean", "sd") %in% names(distr_param_values))) { # transform
-                     distr_param_values$meanlog <- log(distr_param_values$mean^2/sqrt(distr_param_values$mean^2 + distr_param_values$sd^2))
-                     distr_param_values$sdlog <- sqrt(log(1 + distr_param_values$sd^2/distr_param_values$mean^2))
-                   }
-                   # Get shape of generated values and indices of available values
-                   generated_values <- (generated_samples + distr_param_values$meanlog[] + distr_param_values$sdlog[])
-                   available_indices <- which(!is.na(generated_values))
-                   generated_values[] <- 1
-                   # Set the shape of the samples and distribution parameters consistently
-                   generated_samples <- generated_samples*generated_values
-                   distr_param_values <- lapply(distr_param_values, function(x) x*generated_values)
-                   if (all(c("mean", "sd") %in% names(distr_param_values))) {
-                     generated_values <- distr_param_values$mean
-                   } else {
-                     generated_values <- array(NA, dim(distr_param_values$meanlog))
-                   }
-                   # Generate finite values (eliminates warnings)
-                   generated_values[available_indices] <- stats::qlnorm(generated_samples[available_indices],
-                                                                        meanlog = distr_param_values$meanlog[available_indices],
-                                                                        sdlog = distr_param_values$sdlog[available_indices])
-                 },
-               beta =
-                 {
-                   if (all(c("mean", "sd") %in% names(distr_param_values))) { # transform
-                     distr_param_values$sd <- array(distr_param_values$sd, dim(as.matrix(distr_param_values$mean*distr_param_values$sd)))
-                     finite_indices <- which(is.finite(distr_param_values$mean*distr_param_values$sd))
-                     # Limit sd to < sqrt(mean*(1 - mean))
-                     distr_param_values$sd[finite_indices] <- pmin(distr_param_values$sd[finite_indices],
-                                                                   sqrt(distr_param_values$mean[finite_indices]*(1 - distr_param_values$mean[finite_indices]))*0.999)
-                     distr_param_values$alpha <- distr_param_values$mean*(distr_param_values$mean*(1 - distr_param_values$mean)/distr_param_values$sd^2 - 1)
-                     distr_param_values$beta <- (1 - distr_param_values$mean)*(distr_param_values$mean*(1 - distr_param_values$mean)/distr_param_values$sd^2 - 1)
-                   }
-                   # Get shape of generated values and indices of available values
-                   generated_values <- (generated_samples + distr_param_values$alpha[] + distr_param_values$beta[])
-                   available_indices <- which(!is.na(generated_values))
-                   generated_values[] <- 1
-                   # Set the shape of the samples and distribution parameters consistently
-                   generated_samples <- generated_samples*generated_values
-                   distr_param_values <- lapply(distr_param_values, function(x) x*generated_values)
-                   generated_values <- array(NA, dim(generated_values))
-                   generated_values[available_indices] <- stats::qbeta(generated_samples[available_indices],
-                                                                       shape1 = distr_param_values$alpha[available_indices],
-                                                                       shape2 = distr_param_values$beta[available_indices])
-                 },
-               triangular =
-                 {
-                   # Get shape of generated values and indices of any expected NAs
-                   generated_values <- (generated_samples + distr_param_values$lower[] +
-                                          distr_param_values$upper[] + distr_param_values$mode[])
-                   finite_indices <- which(is.finite(generated_values))
-                   generated_values[] <- 1
-                   # Set the shape of the samples and distribution parameters consistently
-                   generated_samples <- generated_samples*generated_values
-                   distr_param_values <- lapply(distr_param_values, function(x) x*generated_values)
-                   # Generate finite values (eliminates NAs which cause qtri errors)
-                   finite_values <- qtri(generated_samples[finite_indices],
-                                                    min = distr_param_values$lower[finite_indices],
-                                                    max = distr_param_values$upper[finite_indices],
-                                                    mode = distr_param_values$mode[finite_indices])
-                   # Correct flat cases (lower >= upper)
-                   flat_indices <- which(distr_param_values$lower[finite_indices] >= distr_param_values$upper[finite_indices])
-                   finite_values[flat_indices] <- distr_param_values$upper[finite_indices][flat_indices]
-                   # Merge the finite values in the expected shape
-                   generated_values[] <- NA
-                   generated_values[finite_indices] <- finite_values
-                 })
+          uniform = generated_values <- stats::qunif(generated_samples,
+            min = distr_param_values$lower[],
+            max = distr_param_values$upper[]
+          ),
+          normal = generated_values <- stats::qnorm(generated_samples,
+            mean = distr_param_values$mean[],
+            sd = distr_param_values$sd[]
+          ),
+          lognormal = {
+            if (all(c("mean", "sd") %in% names(distr_param_values))) { # transform
+              distr_param_values$meanlog <- log(distr_param_values$mean^2 / sqrt(distr_param_values$mean^2 + distr_param_values$sd^2))
+              distr_param_values$sdlog <- sqrt(log(1 + distr_param_values$sd^2 / distr_param_values$mean^2))
+            }
+            # Get shape of generated values and indices of available values
+            generated_values <- (generated_samples + distr_param_values$meanlog[] + distr_param_values$sdlog[])
+            available_indices <- which(!is.na(generated_values))
+            generated_values[] <- 1
+            # Set the shape of the samples and distribution parameters consistently
+            generated_samples <- generated_samples * generated_values
+            distr_param_values <- lapply(distr_param_values, function(x) x * generated_values)
+            if (all(c("mean", "sd") %in% names(distr_param_values))) {
+              generated_values <- distr_param_values$mean
+            } else {
+              generated_values <- array(NA, dim(distr_param_values$meanlog))
+            }
+            # Generate finite values (eliminates warnings)
+            generated_values[available_indices] <- stats::qlnorm(generated_samples[available_indices],
+              meanlog = distr_param_values$meanlog[available_indices],
+              sdlog = distr_param_values$sdlog[available_indices]
+            )
+          },
+          beta = {
+            if (all(c("mean", "sd") %in% names(distr_param_values))) { # transform
+              distr_param_values$sd <- array(distr_param_values$sd, dim(as.matrix(distr_param_values$mean * distr_param_values$sd)))
+              finite_indices <- which(is.finite(distr_param_values$mean * distr_param_values$sd))
+              # Limit sd to < sqrt(mean*(1 - mean))
+              distr_param_values$sd[finite_indices] <- pmin(
+                distr_param_values$sd[finite_indices],
+                sqrt(distr_param_values$mean[finite_indices] * (1 - distr_param_values$mean[finite_indices])) * 0.999
+              )
+              distr_param_values$alpha <- distr_param_values$mean * (distr_param_values$mean * (1 - distr_param_values$mean) / distr_param_values$sd^2 - 1)
+              distr_param_values$beta <- (1 - distr_param_values$mean) * (distr_param_values$mean * (1 - distr_param_values$mean) / distr_param_values$sd^2 - 1)
+            }
+            # Get shape of generated values and indices of available values
+            generated_values <- (generated_samples + distr_param_values$alpha[] + distr_param_values$beta[])
+            available_indices <- which(!is.na(generated_values))
+            generated_values[] <- 1
+            # Set the shape of the samples and distribution parameters consistently
+            generated_samples <- generated_samples * generated_values
+            distr_param_values <- lapply(distr_param_values, function(x) x * generated_values)
+            generated_values <- array(NA, dim(generated_values))
+            generated_values[available_indices] <- stats::qbeta(generated_samples[available_indices],
+              shape1 = distr_param_values$alpha[available_indices],
+              shape2 = distr_param_values$beta[available_indices]
+            )
+          },
+          triangular = {
+            # Get shape of generated values and indices of any expected NAs
+            generated_values <- (generated_samples + distr_param_values$lower[] +
+              distr_param_values$upper[] + distr_param_values$mode[])
+            finite_indices <- which(is.finite(generated_values))
+            generated_values[] <- 1
+            # Set the shape of the samples and distribution parameters consistently
+            generated_samples <- generated_samples * generated_values
+            distr_param_values <- lapply(distr_param_values, function(x) x * generated_values)
+            # Generate finite values (eliminates NAs which cause qtri errors)
+            finite_values <- qtri(generated_samples[finite_indices],
+              min = distr_param_values$lower[finite_indices],
+              max = distr_param_values$upper[finite_indices],
+              mode = distr_param_values$mode[finite_indices]
+            )
+            # Correct flat cases (lower >= upper)
+            flat_indices <- which(distr_param_values$lower[finite_indices] >= distr_param_values$upper[finite_indices])
+            finite_values[flat_indices] <- distr_param_values$upper[finite_indices][flat_indices]
+            # Merge the finite values in the expected shape
+            generated_values[] <- NA
+            generated_values[finite_indices] <- finite_values
+          }
+        )
 
         # Normalize with threshold
         if (is.numeric(normalize_threshold)) {
-          generated_values <- generated_values/normalize_threshold
+          generated_values <- generated_values / normalize_threshold
           generated_values[which(generated_values > 1)] <- 1
         }
 
@@ -672,7 +725,6 @@ Generator <- R6Class("Generator",
         return(TRUE)
       }
     }
-
   ), # end public
 
   private = list(
@@ -684,9 +736,11 @@ Generator <- R6Class("Generator",
     # .region            [inherited]
 
     # Attributes accessible via model get/set methods #
-    .active_attributes = c("region", "coordinates", "description", "inputs", "outputs", "file_templates", "function_templates",
-                           "distribution_templates", "uses_correlations", "spatial_correlation", "temporal_correlation",
-                           "time_steps", "generate_rasters", "decimals", "occupancy_mask", "template_attached"),
+    .active_attributes = c(
+      "region", "coordinates", "description", "inputs", "outputs", "file_templates", "function_templates",
+      "distribution_templates", "uses_correlations", "spatial_correlation", "temporal_correlation",
+      "time_steps", "generate_rasters", "decimals", "occupancy_mask", "template_attached"
+    ),
 
     # Dynamic attributes #
     # .attribute_aliases [inherited]
@@ -698,7 +752,6 @@ Generator <- R6Class("Generator",
     # Errors and warnings #
     # .error_messages    [inherited]
     # .warning_messages  [inherited]
-
   ), # end private
 
   # Active binding accessors for private attributes (above and template nested) #
@@ -885,6 +938,8 @@ Generator <- R6Class("Generator",
             value <- readRDS(file = value)
           } else if (length(grep(".GRD", toupper(value), fixed = TRUE))) {
             value <- raster::brick(value)
+          } else if (length(grep(".TIF", toupper(value), fixed = TRUE))) {
+            value <- raster::brick(value)
           } else {
             value <- utils::read.table(file = value)
           }
@@ -967,6 +1022,5 @@ Generator <- R6Class("Generator",
         super$warning_messages <- value
       }
     }
-
   ) # end active
 )
