@@ -3,6 +3,34 @@
 #' Modular functions for the population simulator for performing a transformation of
 #' the stage abundance (and optionally carrying capacity) at a specified time step via
 #' a user-defined function.
+#' 
+#' @examples 
+#' simulator <- SimulatorReference$new()
+#' # Example transformation: a random population is chosen for a severe disturbance event
+#' # (only life cycle stage 3 individuals survive)
+#' disturbance_function <- function(params) {
+#'   params$simulator$attached$params <- params # attach to reference object
+#'   random_population <- sample(params$occupied_indices, 1)
+#'   new_stage_abundance <- params$stage_abundance
+#'   new_stage_abundance[1:2, random_population] <- 0
+#'   return(new_stage_abundance)
+#' }
+#' transformation_function <- population_transformation(
+#'   replicates = 4, time_steps = 10, years_per_step = 1,
+#'   populations = 7, demographic_stochasticity = TRUE,
+#'   density_stages = c(0, 1, 1), transformation = disturbance_function,
+#'   simulator
+#' )
+#' carrying_capacity <- rep(10, 7)
+#' carrying_capacity <- rep(10, 7)
+#' stage_abundance <- matrix(c(
+#'  7, 13, 0, 26, 0, 39, 47,
+#'  2, 0, 6, 8, 0, 12, 13,
+#'  0, 3, 4, 6, 0, 9, 10
+#' ), nrow = 3, ncol = 7, byrow = TRUE)
+#' occupied_indices <- (1:7)[-5]
+#' transformation_function(r = 2, tm = 6, carrying_capacity, stage_abundance,
+#'                         occupied_indices)
 #'
 #' @param replicates Number of replicate simulation runs.
 #' @param time_steps Number of simulation time steps.
@@ -50,9 +78,7 @@ population_transformation <- function(replicates,
                                       transformation,
                                       simulator,
                                       name = "transformation") {
-
   if (!is.null(transformation)) {
-
     # Derive stages
     stages <- length(density_stages)
 
@@ -65,16 +91,19 @@ population_transformation <- function(replicates,
     }
 
     # List of parameters to pass to the user-defined function
-    params <- c(list(replicates = replicates, time_steps = time_steps, years_per_step = years_per_step,
-                     populations = populations, stages = stages, demographic_stochasticity = demographic_stochasticity,
-                     density_stages = density_stages, simulator = simulator),
-                additional_attributes)
+    params <- c(
+      list(
+        replicates = replicates, time_steps = time_steps, years_per_step = years_per_step,
+        populations = populations, stages = stages, demographic_stochasticity = demographic_stochasticity,
+        density_stages = density_stages, simulator = simulator
+      ),
+      additional_attributes
+    )
 
     if (is.function(transformation)) { # user-defined function
 
       ## Create a nested function for applying user-defined transformation to stage abundance ##
       user_defined_function <- function(r, tm, carrying_capacity, stage_abundance, occupied_indices) {
-
         # Add attributes to be made available to the user-defined function
         params$r <- r
         params$tm <- tm
@@ -83,18 +112,22 @@ population_transformation <- function(replicates,
         params$occupied_indices <- occupied_indices
 
         # Run user-defined transformation function
-        tryCatch({
-          transformed <- transformation(params)
-        },
-        error = function(e){
-          stop(paste("Error produced within user-defined", name, "function:", as.character(e)), call. = FALSE)
-        })
+        tryCatch(
+          {
+            transformed <- transformation(params)
+          },
+          error = function(e) {
+            stop(paste("Error produced within user-defined", name, "function:", as.character(e)), call. = FALSE)
+          }
+        )
 
         # Resolve and check returned transformed values
         if (is.list(transformed)) {
           if (!all(c("stage_abundance", "carrying_capacity") %in% names(transformed))) {
-            stop(paste("When returning a list, the user-defined", name,
-                       "function should contain stage_abundance and carrying_capacity"), call. = FALSE)
+            stop(paste(
+              "When returning a list, the user-defined", name,
+              "function should contain stage_abundance and carrying_capacity"
+            ), call. = FALSE)
           }
         } else { # assume stage abundance matrix and place a list
           transformed <- list(stage_abundance = transformed)
@@ -111,7 +144,7 @@ population_transformation <- function(replicates,
           warning(paste("Negative stage abundances returned by user-defined", name, "function"), call. = FALSE)
         }
         if ("carrying_capacity" %in% names(transformed) &&
-            any(transformed$carrying_capacity[which(is.finite(transformed$carrying_capacity))] < 0)) {
+          any(transformed$carrying_capacity[which(is.finite(transformed$carrying_capacity))] < 0)) {
           warning(paste("Negative carrying capacities returned by user-defined", name, "function"), call. = FALSE)
         }
 
