@@ -1,3 +1,3 @@
-test_abc_function = function (param) {
+test_abc_function <- function(param) {
   return(min(param, na.rm = TRUE))
 }
